from flask import Blueprint, render_template, redirect, url_for, request
from flask_login import login_user, logout_user, login_required, current_user
from sqlalchemy.exc import IntegrityError
from models import db, User
from utils import get_client_ip
from models import RegistrationForm, LoginForm, PasswordField, RecaptchaField, SubmitField
from flask_bcrypt import Bcrypt
from wtforms.validators import DataRequired, Length, EqualTo
from config import Config
auth_bp = Blueprint('auth', __name__)
bcrypt = Bcrypt()

password = PasswordField('Password', validators=[DataRequired(), Length(min=6)])
confirm_password = PasswordField('Confirm Password', validators=[DataRequired(), EqualTo('password')])
recaptcha = RecaptchaField()
submit = SubmitField('Register')

@auth_bp.route('/register', methods=['GET', 'POST'])
def register():
    form = RegistrationForm()

    if form.validate_on_submit():
        hashed_password = bcrypt.generate_password_hash(form.password.data).decode('utf-8')
        ip_address = get_client_ip()
        username = form.username.data.lower()

        existing_user = User.query.filter_by(ip_address=ip_address).first()
        user = User(username=username, encrypted_password=hashed_password, ip_address=ip_address)

        try:
            db.session.add(user)
            db.session.commit()
            return redirect(url_for('auth.login'))
        except IntegrityError:
            db.session.rollback()

    if request.headers.get('X-Requested-With') == 'XMLHttpRequest':
        return render_template('register-modal.html', form=form)

    return render_template('register.html', form=form, recaptcha_key=Config.RECAPTCHA_PUBLIC_KEY)


@auth_bp.route('/login', methods=['GET', 'POST'])
def login():
    form = LoginForm()

    if form.validate_on_submit():
        user = User.query.filter_by(username=form.username.data).first()
        if user and user.check_password(form.password.data):
            login_user(user)
            if user.ip_address is None:
                ip_address = get_client_ip()
                user.ip_address = ip_address
                db.session.commit()
            return redirect(url_for('profile', username=user.username))

    if request.headers.get('X-Requested-With') == 'XMLHttpRequest':
        return render_template('login-modal.html', form=form, recaptcha_key=Config.RECAPTCHA_PUBLIC_KEY)

    return render_template('login.html', form=form, recaptcha_key=Config.RECAPTCHA_PUBLIC_KEY)



@auth_bp.route('/register-modal')
def register_modal():
    form = RegistrationForm()
    return render_template('register-modal.html', form=form)

@auth_bp.route('/login-modal')
def login_modal():
    form = LoginForm()
    return render_template('login-modal.html', form=form)


@auth_bp.route('/logout')
def logout():
    logout_user()
    return redirect(url_for('index'))